/**
*
* $Id: CVEL-VBMOON.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from von Braun Colliers article and other von Braun books
const double ISP_FIRST_SL		=     297.0*G; 
const double ISP_FIRST_VAC		=     297.0*G; //***
const double ISP_GYRO			=      10e9*G;
const double THRUST_FIRST_VAC	=   13574.0*G; //***
const double MAX_ATT_FIRST		=     800.0*G;
const double FIRST_PROP_MASS	= 3136000.0; //*** 
const double SECOND_PROP_MASS	=  870000.0;
const double FOURTH_PROP_MASS	=   79000.0;
const double RESERVE_PROP_MASS	=    6400.0;
const double THIRD_PROP_MASS	=  215000.0 + FOURTH_PROP_MASS + RESERVE_PROP_MASS;
const double VBMOON_DRYMASS		=   32800.0; //***
const double BIGTANKS_DRYMASS	=   14000.0; //***
const double SMALLTANKS_DRYMASS	=   17000.0; //***

const VECTOR3 VBMOON_OFS		= _V(0,0,0);
const VECTOR3 ASTROLANDED_OFS	= _V( 5.00, 0.0,-31.5);
const VECTOR3 ASTRO_OFS			= _V( 1.45, 0.0,  5.7);

const double GEAR_OPERATING_SPEED = 0.125;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double BRAKES_OPERATING_SPEED = 0.25;
const double HATCH_OPERATING_SPEED = 0.125;

#define PLAYGEAR					 20		// the gear sound
#define PLAYBURN					 24		// the Burn Now sound
#define PLAYFIVE					 25     // the "5 seconds" call

/* make your own vessel class (named VBMOON here, for example)
   subclassed from COMMON_VESSEL_EX */

class VBMOON : public COMMON_VESSEL_EX {
public:
	VBMOON (OBJHANDLE hObj, int fmodel);
	~VBMOON();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, pldoors_status, brakes_status, hatch_status;

	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void SeparateEVA();
	void SetGearParameters (double state);
	void JettisonLegs();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void VectorThrust();
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 
	void MakeThrusters(PROPELLANT_HANDLE ph_TLI);
	void SetStage0();
	void SetStage1();
	void SetStage2();
	void JettisonPart();
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	int MyID;

// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

protected:
	PROPELLANT_HANDLE 	ph_TLI, ph_LAND, ph_TEI, ph_DOCK, ph_RESV;
	THRUSTER_HANDLE   th_first[30];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hVBMOON, hVBMARS, hCtrTanks, hCargoCyl, hPropTank1, hPropTank2, hSideTank1, hSideTank2;
	MESHHANDLE        hLeg1, hLeg2, hLeg3, hLeg4, hLeg5;
	SURFHANDLE		  tex,texRCS, retex;
	int stage;
//	UINT anim_lrudder,anim_rrudder,anim_laileron,anim_raileron,anim_lelevator,anim_relevator,anim_lelevatortrim,anim_relevatortrim;
	UINT anim_gear;     // handle for landing gear animation
//	UINT anim_pldoors;     // handle for payload door animation
	double gear_proc;
//	double pldoors_proc;
	bool mars, cargo, bEVA;
//	UINT Exh1,Exh2,Exh3,Exh4;
//	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
//	VECTOR3 Vold, Vnew;
//	double dt, oldsimt;
//	ATTACHMENTHANDLE Baypos;
//	VECTOR3 gpos, grms, pos, dir, rot;
//	OBJHANDLE hV;
//	DWORD i, j;
//	double atchmass;
//	DOCKHANDLE hDock;
	bool JettLegsArm;
	int Sweepnum;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	double dV;
	double mdot;
	double BurnTime, FallTime;
	VECTOR3 Rel_Vel;
	double pitchangle;
	DOCKHANDLE dock0, dock1, dock2;
//	OBJHANDLE hMoon;
	ANIMATIONCOMPONENT_HANDLE hG1,hG11,hG2,hG21,hG3,hG31,hG4,hG41,hG51,hG52,hG53;
};







/* basic vessel class setup */

void VBMOON::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void VBMOON::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void VBMOON::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void VBMOON::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void VBMOON::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void VBMOON::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int VBMOON::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new VBMOON (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (VBMOON*)vessel;
}

